@@ -1,5 +1,5 @@
 /* Tests of access.
-   Copyright (C) 2019-2023 Free Software Foundation, Inc.
+   Copyright (C) 2019-2025 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -36,5 +36,5 @@ main ()
 {
   test_access (access);
 
-  return 0;
+  return test_exit_status;
 }
