@@ -1,5 +1,5 @@
 /* GNU gettext - internationalization aids
-   Copyright (C) 1995-2024 Free Software Foundation, Inc.
+   Copyright (C) 1995-2025 Free Software Foundation, Inc.
 
    This file was written by Peter Miller <millerp@canb.auug.org.au>
 
@@ -47,11 +47,16 @@ const char *const format_language[NFORMATS] =
   /* format_lisp */             "lisp",
   /* format_elisp */            "elisp",
   /* format_librep */           "librep",
+  /* format_rust */             "rust",
+  /* format_go */               "go",
   /* format_ruby */             "ruby",
   /* format_sh */               "sh",
+  /* format_sh_printf */        "sh-printf",
   /* format_awk */              "awk",
   /* format_lua */              "lua",
   /* format_pascal */           "object-pascal",
+  /* format_modula2 */          "modula2",
+  /* format_d */                "d",
   /* format_smalltalk */        "smalltalk",
   /* format_qt */               "qt",
   /* format_qt_plursl */        "qt-plural",
@@ -82,11 +87,16 @@ const char *const format_language_pretty[NFORMATS] =
   /* format_lisp */             "Lisp",
   /* format_elisp */            "Emacs Lisp",
   /* format_librep */           "librep",
+  /* format_rust */             "Rust",
+  /* format_go */               "Go",
   /* format_ruby */             "Ruby",
   /* format_sh */               "Shell",
+  /* format_sh_printf */        "Shell printf",
   /* format_awk */              "awk",
   /* format_lua */              "Lua",
   /* format_pascal */           "Object Pascal",
+  /* format_modula2 */          "Modula-2",
+  /* format_d */                "D",
   /* format_smalltalk */        "Smalltalk",
   /* format_qt */               "Qt",
   /* format_qt_plural */        "Qt plural",
@@ -504,7 +514,7 @@ message_list_copy (message_list_ty *mlp, int copy_level)
 
 
 message_ty *
-message_list_search (message_list_ty *mlp,
+message_list_search (const message_list_ty *mlp,
                      const char *msgctxt, const char *msgid)
 {
   if (mlp->use_hashtable)
