@@ -1,5 +1,5 @@
 /* Reading NeXTstep/GNUstep .strings files.
-   Copyright (C) 2003, 2006, 2015 Free Software Foundation, Inc.
+   Copyright (C) 2003-2025 Free Software Foundation, Inc.
    Written by Bruno Haible <bruno@clisp.org>, 2003.
 
    This program is free software: you can redistribute it and/or modify
@@ -13,14 +13,26 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 #ifndef _READ_STRINGTABLE_H
 #define _READ_STRINGTABLE_H
 
 #include "read-catalog-abstract.h"
 
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+
 /* Describes a .strings file parser.  */
-extern DLL_VARIABLE const struct catalog_input_format input_format_stringtable;
+extern LIBGETTEXTSRC_DLL_VARIABLE const struct catalog_input_format input_format_stringtable;
+
+
+#ifdef __cplusplus
+}
+#endif
+
 
 #endif /* _READ_STRINGTABLE_H */
