@@ -1,6 +1,5 @@
 /* GNU gettext - internationalization aids
-   Copyright (C) 1995, 1998, 2000-2004, 2006, 2009, 2020 Free Software
-   Foundation, Inc.
+   Copyright (C) 1995-2024 Free Software Foundation, Inc.
 
    This file was written by Peter Miller <millerp@canb.auug.org.au>
 
@@ -75,6 +74,26 @@ string_list_append (string_list_ty *slp, const char *s)
 }
 
 
+/* Append a freshly allocated single string to the end of a list of strings,
+   transferring its ownership to SLP.  */
+void
+string_list_append_move (string_list_ty *slp, char *s)
+{
+  /* Grow the list.  */
+  if (slp->nitems >= slp->nitems_max)
+    {
+      size_t nbytes;
+
+      slp->nitems_max = slp->nitems_max * 2 + 4;
+      nbytes = slp->nitems_max * sizeof (slp->item[0]);
+      slp->item = (const char **) xrealloc (slp->item, nbytes);
+    }
+
+  /* Add the string itself to the end of the list.  */
+  slp->item[slp->nitems++] = s;
+}
+
+
 /* Append a single string to the end of a list of strings, unless it is
    already contained in the list.  */
 void
@@ -82,7 +101,7 @@ string_list_append_unique (string_list_ty *slp, const char *s)
 {
   size_t j;
 
-  /* Do not if the string is already in the list.  */
+  /* Do nothing if the string is already in the list.  */
   for (j = 0; j < slp->nitems; ++j)
     if (strcmp (slp->item[j], s) == 0)
       return;
@@ -100,6 +119,37 @@ string_list_append_unique (string_list_ty *slp, const char *s)
   slp->item[slp->nitems++] = xstrdup (s);
 }
 
+/* Likewise with a string descriptor as argument.  */
+void
+string_list_append_unique_desc (string_list_ty *slp,
+                                const char *s, size_t s_len)
+{
+  size_t j;
+
+  /* Do nothing if the string is already in the list.  */
+  for (j = 0; j < slp->nitems; ++j)
+    if (strlen (slp->item[j]) == s_len && memcmp (slp->item[j], s, s_len) == 0)
+      return;
+
+  /* Grow the list.  */
+  if (slp->nitems >= slp->nitems_max)
+    {
+      slp->nitems_max = slp->nitems_max * 2 + 4;
+      slp->item = (const char **) xrealloc (slp->item,
+                                            slp->nitems_max
+                                            * sizeof (slp->item[0]));
+    }
+
+  /* Add a copy of the string to the end of the list.  */
+  {
+    char *copy = XNMALLOC (s_len + 1, char);
+    memcpy (copy, s, s_len);
+    copy[s_len] = '\0';
+
+    slp->item[slp->nitems++] = copy;
+  }
+}
+
 
 /* Destroy a list of strings.  */
 void
@@ -236,6 +286,18 @@ string_list_member (const string_list_ty *slp, const char *s)
   return false;
 }
 
+/* Likewise with a string descriptor as argument.  */
+bool
+string_list_member_desc (const string_list_ty *slp, const char *s, size_t s_len)
+{
+  size_t j;
+
+  for (j = 0; j < slp->nitems; ++j)
+    if (strlen (slp->item[j]) == s_len && memcmp (slp->item[j], s, s_len) == 0)
+      return true;
+  return false;
+}
+
 
 /* Remove s from the list of strings.  Return the removed string or NULL.  */
 const char *
